c  ****************************************************************************
c  *                                                                          *
c  *                  *****     subroutine get_tpo_g     *****                *
c  *                                                                          *
c  *    determine theta, phi, and omega for current image.                    *
c  *                                                                          *
c  *    protocol:                                                             *
c  *     1. compute polar image from raw (cartesian) image (r4_img->r4_pimg). *
c  *     2. compute polar fft (pfft) from r4_pimg.                            *
c  *     3. convert image pft to amplitudes & extract desired portion.        *
c  *     4. find average value of image pft (also termed particle pft).       *
c  *     5. cross-correlate particle pft with all reference pfts to get theta *
c  *         and phi (|phi| for n22 and 532 symmetry, both theta and phi may  *
c  *         change if the object is asymmetric or n-fold.                    *
c  *     6. read in the appropriate projection (prj) corresponding to theta   *
c  *         and |phi|.                                                       *
c  *     7. convert prj to polar projection (polar_prj).                      *
c  *     8. cross-correlate polar_prj with r4_pimg to determine sign of       *
c  *         phi and value of omega (also theta if sym=1-20).                 *
c  *                                                                          *
c  *    february 1998 purdue university alphastation 500/400, t. baker.       *
c  *                                                                          *
c  *    modified  6-feb-1998 by t. baker (extracted subs from pftcc).         *
c  *    last modified 28-apr-1998 by t. baker (global specific).              *
c  ****************************************************************************
c
      subroutine get_tpo_g (mode, ncol, nrow, nrot, nannuli, nview_mod, 
     1     jcut, xcen, ycen, xcen_model, 
     1     ycen_model, radius, del_ang, 
     2     pixres_min, pixres_max, ann_lo, ann_hi, var, avg, 
     3     sq1, sq2, mod_ts, mod_ps, the_best, phi_best, ome_best, 
     4     pfts, pft_avgs, prjs, cc_pft, cc_prj, 
     5     r4_img, r4_pimg, pfft, prj, polar_prj, pftimg, 
     6     flip, index)

      implicit none

      integer*4 ncol, nrow, nrot, nannuli, jcut, mode
      integer*4 ann_lo, ann_hi, nview_mod, index
      integer*4 i, j
      real*4    xcen, ycen, xcen_model, ycen_model
      real*4    radius, del_ang, pixres_min, pixres_max, cc_pft, cc_prj
      real*4    pi, avg, sq1, the_best, phi_best, ome_best
      real*4    mod_ts(nview_mod), mod_ps(nview_mod), sq2(nview_mod)
      real*4    r4_img(ncol,nrow), prj(ncol,nrow), r4_pimg(nrot,nannuli)
      real*4    prjs(ncol,nrow,nview_mod), polar_prj(nrot,nannuli)
      real*4    pfft(2*nrot,nannuli), pfts(nrot/4,nannuli,nview_mod)
      real*4    pft_avgs(nview_mod)
      real*4    pftimg((nrot/4),nannuli)
      logical   flip, var(nrot/4,nannuli)
c ----------------------------------------------------------------------

      pi = acos(-1.0)

      call map_polar (r4_img, ncol, nrow, r4_pimg, nrot, nannuli, xcen, 
     1 ycen, 1.0, radius, del_ang)

      call pmap_fft (r4_pimg, pfft, nrot, nannuli)

      do i = 1, nannuli             ! convert pfft a & b parts to amplitudes
          do j = 1,(nrot/4)
          pftimg(j,i) = sqrt (pfft(2*j-1,i)**2 + pfft(2*j,i)**2)
        enddo
      enddo

      call avg_pftimg (pftimg, nrot, nannuli, jcut, pixres_min, 
     1 pixres_max, ann_lo, ann_hi, var, avg, sq1)

      call get_thephi (pftimg,pfts,pft_avgs,nrot,nannuli,nview_mod, 
     1 sq2,jcut,pixres_min,pixres_max,cc_pft,ann_lo,ann_hi, 
     2 var,avg,sq1,index)

      the_best = mod_ts(index)
      phi_best = mod_ps(index)

c      call get_bestprj (prj, ncol, nrow, nview_mod, prjs, mode, 
c     1 index, prjhead)
c     replaces call to get_bestprj - assuming that mode >= 0
c     and prjs had already been filled by calc_pfts_g
      prj(:,:) = prjs(:,:,index) !f90 array syntax

      call map_polar (prj, ncol, nrow, polar_prj, nrot, nannuli, 
     1 xcen_model, ycen_model, 1.0, radius, del_ang)

      call get_phiomega (r4_pimg, polar_prj, nrot, nannuli, del_ang, 
     1 ome_best, ann_lo, ann_hi, flip, cc_prj)

      return
      end subroutine get_tpo_g
